moreThanXTraces = function(data,x) {

  attachers.trace.counts=aggregate(cbind(data$attacher), 
    by=list(data$attacher), function(x) {length(x)})
  names(attachers.trace.counts)[names(attachers.trace.counts)=="Group.1"]="attacher";
  names(attachers.trace.counts)[names(attachers.trace.counts)=="V1"]="number.of.traces";

  more.than.x=subset(attachers.trace.counts,attachers.trace.counts$number.of.traces>=x);
  more.than.x
}


######################### graphs ####################################

histogramsAttachers <- function(more.than.5) {

  par(xpd=T, mar=par()$mar+c(-1,-0.3,-4,-2.0)) # trim
  h=hist(more.than.5$number.of.traces,breaks=800,#c(0,10,20,30,40,50,60,70,80,90,150,200,300,400,800),
      freq=TRUE,
      #ylim=c (min (1:length (attachers.trace.counts[["number.of.traces"]])), 40),
     # xlim=c(5,400),
       ylab="# of programmers contributed to the given # of traces", 
       xlab="Number of traces >= 5", main="",
	  col="gray40",border="white",ylim=c(0,20),xlim=c(5,800)
  );
  h$mid=h$mid-0.5;
  plot(h$mid,h$counts,log="x",col="gray20",type="h",
    xlab="Number of traces >= 5 (in log scale)",
    ylab="# of programmers");

  boxplot(more.than.5$number.of.traces,outline=FALSE,
      horizonal=FALSE,
       main="Distribution of number of traces programmers contribute to");
  subset(more.than.5,more.than.5$number.of.traces>200)

  setwd("../analyze-using-probability-beginning-or-end");
  dev.print(device=postscript,file="histogram-attachers-number-of-traces2.eps",
    onefile=FALSE, paper="special", horizontal=FALSE);

}

histogramEventTiming <-function(events) {

  edits=subset(events,events$kind=="edit");
  selects=subset(events,events$kind=="selection");

  par(mfrow=c(2,1),oma=c(0,0,1,0.5), # oma: bottom,left,top,right
    mar=c(5, 4, 1, 0)) # mar: bottom,left,top,right

  numbers <- hist(edits$start.date,plot=FALSE);  
  numbers$counts <- numbers$counts/1000;
  plot(numbers$mids,numbers$counts,
       col="blue", type="o",
       ylab="Frequency (in thousnads)", xlab="Start time of events", main="Start time of events");
  numbers2 <- hist(selects$start.date,plot=FALSE);
  numbers2$counts <- numbers2$counts/1000;  
  lines(numbers2$mids,numbers2$counts,type="o", pch=22, lty=2, col="red")
  legend("top", c("edits", "selects"), cex=0.8, bty="n",
         col=c("blue","red"), pch=21:22, lty=1:2);

  numbers <- hist(edits$start.date,plot=FALSE);
  numbers$counts <- numbers$counts/1000;
  plot(numbers$mids,numbers$counts,
       col="blue", type="o",
       ylab="Frequency (in thousands)", xlab="End time of events", main="End time of events")
  numbers2 <- hist(selects$end.date,plot=FALSE);
  numbers2$counts <- numbers2$counts/1000;  
  lines(numbers2$mids,numbers2$counts,type="o", pch=22, lty=2, col="red")
  legend("top", c("edits", "selects"), cex=0.8, bty="n",
         col=c("blue","red"), pch=21:22, lty=1:2);

  #mtext("Event timing within tasks", NORTH<-3, line=2, adj=0.5, cex=1.2, outer=TRUE);

  setwd("../analyze-using-probability-beginning-or-end");
  dev.print(device=postscript,file="histograms-event-timing.eps",
    onefile=FALSE, paper="special", horizontal=FALSE);
  dev.off();
}

histogramEventDiffs=function(events) {

  edits<-subset(events,events$kind=="edit");
  diff=events$end.date-events$start.date
  diff= subset(diff,diff>=0);

  print("eliminate diffs with < 0 diffs:");
  (length(subset(diff,diff<0)))
  par(xpd=T, mar=par()$mar+c(-1,0,-3.7,-1.7)) # trim
  h=hist(diff,breaks=300,plot=FALSE);
  plot(h$mid,h$counts,log="y",col="gray",type="h",
    xlab="Duration of an edit, normalized by the total duration of the trace",
    ylab="Frequency in log scale",yaxt="n",pch=21);
  axis(2,at=c(100,1000,10000),cex.axis=0.8);
  axis(2,at=c(100000,1000000),cex.axis=0.8);

  setwd("../analyze-using-probability-beginning-or-end");
  dev.print(device=postscript,file="histogram-duration-edits.eps",
    onefile=FALSE, paper="special", horizontal=FALSE);
  dev.off();

}

histogramPercentage=function(traces) {
  par(xpd=T, mar=par()$mar+c(-1.1,-1.2,-4.1,-2.1)) # trim
  h=hist(traces$percentage.of.edits.first.half,freq=FALSE,breaks=20,
    xlab=expression(~"Fr("~e[t]~"= first ): Fraction of edits in the first half of the trace t"),
    ylab="",
    border=TRUE, main="",col="gray80", yaxt="n");
  #axis(1,at=c(200,400,600),labels=c("200","400","600"));
  d<-density(traces$percentage.of.edits.first.half)
  lines(d$x,d$y,col="red",lty=2, lwd=3)

  setwd("../analyze-using-probability-beginning-or-end");
  dev.print(device=postscript,file="histogram-fraction-of-edits-in-first-half-of-trace2.eps",
    onefile=FALSE, paper="special", horizontal=FALSE);
  dev.off();
  
}

mosiacPlotForEditingStyles=function(inputData,c1,c2) {
  tbl=data.frame(inputData[[c1]],inputData[[c2]]);
  names(tbl)=c(c1,c2);
  (tbl=table(tbl))
  print("generating mosaic graph");
  library(vcd)
  mosaic(tbl,shade=TRUE,legend=TRUE);
}

stackedBarPlotProportionOfEditingStyles=function(data) {
  counts=table(data$editing.style,data$attacher);
  counts=counts[,order(colSums(counts), decreasing=TRUE)]
  counts=counts[,1:20];
  xaxis="Top 20 programmers in terms of number of traces contributed";
  colours=c("black","gray80","gray80");
  par(xpd=T, mar=par()$mar+c(-3.2,-0.1,-3.2,-1.8)) # trim
  barplot(counts,ylim=c(0,800),col=colours,
    density=c(-1,-1,15),  angle=c(-1,-1,45),border=TRUE,
    xaxt="n", main="", ylab="Number of traces with certain editing styles")
  legend("top",legend=rownames(counts),fill=colours,
    density=c(-1,-1,20),  angle=c(-1,-1,45),border=TRUE,
    bty="n");
  mtext(xaxis, SOUTH<-1, at=12, line=0.5)

  setwd("../analyze-using-probability-beginning-or-end");
  dev.print(device=postscript,file="stacked-bar-plot-editing-styles.eps",
    onefile=FALSE, paper="special", horizontal=FALSE);

  dev.off();
}

stackedBarPlotEditingStylesBySeverity=function(data){

  counts =table(data$editing.style, data$bug.severity)
  counts[1:3]=counts[1:3]/sum(counts[1:3])
  counts[4:6]=counts[4:6]/sum(counts[4:6])
  counts[7:9]=counts[7:9]/sum(counts[7:9])
  colors <- c("white","grey40","grey80")
  par(xpd=T, mar=par()$mar+c(-3,0,-3,6)) # expand the right side for the legend
  barplot(counts, ylab="Fraction of editing styles for tasks",
    beside=FALSE,col=colors)
  legend(3.6,0.7,
    legend=c("edit-first","edit-last","edit-throughout"),#rownames(counts),
    fill=colors, bty="n");
  setwd("../analyze-using-probability-beginning-or-end");
  dev.print(device=postscript,file="stacked-barplot-editing-style-type-percentages.eps",
    onefile=FALSE, paper="special", horizontal=FALSE);
}

boxplotPlotProgrammingStylesBySeverity=function(attacher.data){

  column="edit.first";
  yLabel="Fraction of edit-first traces";
  filename="boxplot-programmer-edit-first-fractions-per-type.eps";
  temp1=data.frame(attacher.data[[column]],attacher.data$enhancement.category);
  names(temp1)=c(column,"task.type");
  temp2=data.frame(attacher.data[[column]],attacher.data$minor.category);
  names(temp2)=c(column,"task.type");
  temp3=data.frame(attacher.data[[column]],attacher.data$normal.category);
  names(temp3)=c(column,"task.type");
  temp = rbind(temp1,temp2,temp3);
  par(xpd=T, mar=par()$mar+c(-3,0,-3.7,-1.7)) # trim
  boxplot(temp[[column]]~temp$task.type,
    boxwex=0.5,fill="gray",xaxt="n",col=c("gray50","gray90"),
    at=c(1.1,1.9,3.1,3.9,5.1,5.9),
    ylab=yLabel,outline=TRUE,ylim=c(0,1))
  axis(1,at=c(1.5, 3.5, 5.5),labels=c("enhancement","minor","major"));
  setwd("../analyze-using-probability-beginning-or-end");
  dev.print(device=postscript,file=filename,
    onefile=FALSE, paper="special", horizontal=FALSE);


  column="edit.last";
  yLabel="Fraction of edit-last traces";
  filename="boxplot-programmer-edit-last-fractions-per-type.eps";
  temp1=data.frame(attacher.data[[column]],attacher.data$enhancement.category);
  names(temp1)=c(column,"task.type");
  temp2=data.frame(attacher.data[[column]],attacher.data$minor.category);
  names(temp2)=c(column,"task.type");
  temp3=data.frame(attacher.data[[column]],attacher.data$normal.category);
  names(temp3)=c(column,"task.type");
  temp = rbind(temp1,temp2,temp3);
  par(xpd=T, mar=par()$mar+c(-3,0,-3.7,-1.7)) # trim
  boxplot(temp[[column]]~temp$task.type,
    boxwex=0.5,fill="gray",xaxt="n",col=c("gray50","gray90"),
    at=c(1.1,1.9,3.1,3.9,5.1,5.9),
    ylab=yLabel,outline=TRUE,ylim=c(0,1))
  axis(1,at=c(1.5, 3.5, 5.5),labels=c("enhancement","minor","major"));
  setwd("../analyze-using-probability-beginning-or-end");
  dev.print(device=postscript,file=filename,
    onefile=FALSE, paper="special", horizontal=FALSE);

  column="edit.throughout";
  yLabel="Fraction of edit-throughout traces";
  filename="boxplot-programmer-edit-throughout-fractions-per-type.eps";
  temp1=data.frame(attacher.data[[column]],attacher.data$enhancement.category);
  names(temp1)=c(column,"task.type");
  temp2=data.frame(attacher.data[[column]],attacher.data$minor.category);
  names(temp2)=c(column,"task.type");
  temp3=data.frame(attacher.data[[column]],attacher.data$normal.category);
  names(temp3)=c(column,"task.type");
  temp = rbind(temp1,temp2,temp3);
  par(xpd=T, mar=par()$mar+c(-0.2,0,-3.7,-1.7)) # expand the bottom side for the legend
  boxplot(temp[[column]]~temp$task.type,
    boxwex=0.5,fill="gray",xaxt="n",col=c("gray50","gray90"),
    at=c(1.1,1.9,3.1,3.9,5.1,5.9),
    ylab=yLabel,outline=TRUE,ylim=c(0,1))
  axis(1,at=c(1.5, 3.5, 5.5),labels=c("enhancement","minor","major"));
  legend(2,-0.25,fill=c("gray50","gray90"),bty="n",
    legend=c("less than median for the task type", "more than median for the task type"))
  setwd("../analyze-using-probability-beginning-or-end");
  dev.print(device=postscript,file=filename,
    onefile=FALSE, paper="special", horizontal=FALSE);
}

boxplotEditingStylesByExpertise=function(attacher.data){
  colour=c("gray50","gray90")
  par(xpd=T, mar=par()$mar+c(0.5,-0.3,-3.9,-1.9)) # expand the bottom side for the legend
  boxplot(attacher.data$edit.first~attacher.data$expert.level,
    boxwex = 0.5, at = c(1.1,1.9),xlim=c(1,6), col=colour,
    yaxt="n",xaxt="n",ylim=c(0,1),ylab="Fractions")
  boxplot(attacher.data$edit.last~attacher.data$expert.level,add=TRUE,
    boxwex = 0.5, at = c(3.1,3.9),col=colour,xaxt="n",yaxt="n");
  boxplot(attacher.data$edit.throughout~attacher.data$expert.level,add=TRUE,
    boxwex = 0.5, at = c(5.1,5.9),col=colour,xaxt="n",yaxt="n")
  axis(1,at=c(1.5,3.5,5.5),labels=c("edit-first","edit-last","edit-throughout"));
  axis(2,at=c(0,0.25,0.5,0.75,1),labels=c("0","0.25","0.5","0.75","1"));
  legend(3,-0.45,fill=colour,bty="n",
    legend=c("less than median for expertise", "more than median for expertise"))
  setwd("../analyze-using-probability-beginning-or-end");
  dev.print(device=postscript,file="boxplot-editing-styles-by-expertise.eps",
    onefile=FALSE, paper="special", horizontal=FALSE);
}

stackedBarPlotEditingStylesByStackTrace=function(data){

  counts =table(data$editing.style, data$contain.stack.trace)
  counts[1:3]=counts[1:3]/sum(counts[1:3])
  counts[4:6]=counts[4:6]/sum(counts[4:6])
  colors <- c("white","grey40","grey80")
  par(xpd=T, mar=par()$mar+c(-2,0,-3,8)) # expand the right side for the legend
  barplot(counts, ylab="Fraction of editing styles for tasks",
    beside=FALSE,col=colors,xaxt="n");
  axis(1,line=1,
    at=c(0.7,1.9),labels=c("contain a stack trace", "does not contain"),tick=FALSE);
  legend(2.4,0.7,
    legend=c("edit-first","edit-last","edit-throughout"),#rownames(counts),
    fill=colors, bty="n");
  setwd("../analyze-using-probability-beginning-or-end");
  dev.print(device=postscript,file="stacked-barplot-editing-style-stack-trace-percentages.eps",
    onefile=FALSE, paper="special", horizontal=FALSE);
}

############################ binning tasks #################################

computeEditingStyleBasedOnEditsInFirstHalf <-function(traces) {

  edit.last=subset(traces,traces$percentage.of.edits.first.half<=0.19);
  edit.first=subset(traces,traces$percentage.of.edits.first.half>=0.87);
  others=subset(traces,traces$percentage.of.edits.first.half>0.19 
    & traces$percentage.of.edits.first.half<0.87);

  edit.first$editing.style="edit.first";
  edit.last$editing.style="edit.last";
  others$editing.style="edit.throughout";

  aggr=rbind(edit.first,edit.last,others);
  names(aggr)=names(edit.first);
  
  aggr1=data.frame(aggr$attachment.id,aggr$editing.style);
  names(aggr1)=c("attachment.id","editing.style");
  aggr1$editing.style=as.factor(aggr1$editing.style);
  aggr1
}

computePercentageOfEditsInBeginning <-function(events) {  # expecting attributes: "attacher" "kind" "attachment.id" "start.date"

  events<-subset(events,events$kind=="edit");

  events$beginningOrEnd=(events$start.date<0.5 & events$end.date<0.5) | ((0.5-events$start.date) > (events$end.date-0.5))
    
  aggr = aggregate(events$beginningOrEnd,by=list(events$attachment.id),
    FUN=collectBeginningStyles)
  names(aggr)=c("attachment.id","percentage.of.edits.first.half");
  aggr$attachment.id=as.factor(aggr$attachment.id);
  aggr
}

collectBeginningStyles <- function(edits) {    # expecting attributes "kind" "start.date", with the same attacher
  numberOfEditEvents=length(edits);
  numberOfEditEventsInFirstHalf = length(which(edits == TRUE))
  (ratioOfEditsInFirstHalf= numberOfEditEventsInFirstHalf / numberOfEditEvents)
}

################### binning personal editing styles #############################

computePersonalEditingStyleBasedOnEditsInFirstHalf <-function(data) {  # expecting attributes: "attacher" "editing.style"

  aggr = aggregate(data$editing.style,by=list(data$attacher),FUN=calculateEditFirstFraction)
  names(aggr)=c("attacher", "edit.first");

  aggr1 = aggregate(data$editing.style,by=list(data$attacher),FUN=calculateEditLastFraction)
  names(aggr1)=c("attacher", "edit.last");
  aggr=merge(aggr,aggr1);

  aggr2 = aggregate(data$editing.style,by=list(data$attacher),FUN=calculateEditThroughoutFraction)
  names(aggr2)=c("attacher", "edit.throughout");
  aggr=merge(aggr,aggr2);

  (mean1 = mean(aggr$edit.first))
  (mean2 = mean(aggr$edit.last))
  (mean3 = mean(aggr$edit.throughout))
  (stdDev1=sd(aggr$edit.first))
  (stdDev2=sd(aggr$edit.last))
  (stdDev3=sd(aggr$edit.throughout))

  aggr$sd1 = abs(aggr$edit.first - mean1)/stdDev1
  aggr$sd2 = abs(aggr$edit.last - mean2)/stdDev2
  aggr$sd3 = abs(aggr$edit.throughout - mean3)/stdDev3

  result1 = aggr[which(sd1> sd2 & sd1 > sd3),]
  result1$personal.editing.style="more.edit.first";
  result2 = aggr[which(sd2> sd1 & sd2 > sd3),]
  result2$personal.editing.style="more.edit.last";
  result3 = aggr[which(sd3> sd1 & sd3 > sd2),]
  result3$personal.editing.style="more.edit.throughout";

  for( i in 1:dim(result1)[1] ) {
    print(i)
    (cst=chisq.test(c(result1[i,]$sd1,result1[i,]$sd2,result1[i,]$sd3)))
    if( cst$p.value < 0.05) {	
      print(cst)
    }
  }

  result=rbind(result1,result2,result3);
  result$personal.editing.style=as.factor(result$personal.editing.style);

  result
}

calculateEditFirstFraction <-function(x) { 
  length(which(x=="edit.first")) / length(x)
}

calculateEditLastFraction =function(x) {
   length(which(x=="edit.last"))  / length(x)
}

calculateEditThroughoutFraction=function(x) {
    length(which(x=="edit.throughout")) / length(x)
}


almostAllOfEditingStyleBasedOnEditsInFirstHalf <-function(x) { 
  result="no.definite.style";
  if( length(which(x=="edit.first")) > length(x)*0.3 ) {
    result="mostly.edit.first";
  } else if( length(which(x=="edit.last")) > length(x)*0.3) {
    result="mostly.edit.last";
  } else if( length(which(x=="edit.throughout")) > length(x)*0.3) {
    result="mostly.edit.throughout";
  }
  result
}

majorityEditingStyle=function(x) {
  editing.styles=c("edit.first","edit.last","edit.throughout");
  choose.max=c(length(which(x==editing.styles[1])),length(which(x==editing.styles[2])),
    length(which(x==editing.styles[3])))
  i=which(choose.max==max(choose.max))
  editing.styles[i]
}

computePersonalEditingStyleBasedOnClustering<-function(data) {  # expecting attributes: "attacher" "editing.style"

  aggr = aggregate(data$editing.style,by=list(data$attacher),FUN=calculateEditFirstFraction)
  names(aggr)=c("attacher", "edit.first");

  aggr1 = aggregate(data$editing.style,by=list(data$attacher),FUN=calculateEditLastFraction)
  names(aggr1)=c("attacher", "edit.last");
  aggr=merge(aggr,aggr1);

  aggr2 = aggregate(data$editing.style,by=list(data$attacher),FUN=calculateEditThroughoutFraction)
  names(aggr2)=c("attacher", "edit.throughout");
  aggr=merge(aggr,aggr2);
  
  result=clusterPersonalEditingStyle(aggr);
  result
}

clusterPersonalEditingStyle<-function(aggr) {
  library(cluster);

  attrs <- names(aggr) %in% c("edit.first","edit.last") 
  d <-aggr[attrs]
  library(fpc)
  result=pamk(daisy(d));
 # numberOfClusters=result$nc;
  fit <- pam(daisy(d), 2) 
  # 2 for edit.first/edit.last
  # 3 for edit.first/edit.throughout

  plot(fit) # for silhouette plot  
  print(aggr[fit$medoids,])
  
  aggr <- data.frame(aggr, fit$cluster)  # append cluster assignment
  names(aggr)[which(names(aggr)=="fit.cluster")] = "personal.editing.style";
  aggr$personal.editing.style=as.factor(aggr$personal.editing.style);
  aggr$personal.editing.style=drop.levels(aggr$personal.editing.style);
  aggr
}

scatterPlotOfClusters=function(editing.style.per.attacher) {
  cluster1=subset(editing.style.per.attacher,editing.style.per.attacher$personal.editing.style==1)[,2:3]
  cluster2=subset(editing.style.per.attacher,editing.style.per.attacher$personal.editing.style==2)[,2:3]
  plot(cluster1,type="p",col="red", xlim=c(0,1),ylim=c(0,1),
    main="Scatter plot of edit-first fraction and edit-last fractions",
    xlab="Edit first fractions",ylab="Edit last fractions")
  points(cluster2, col="blue",pch=24)
}

computePersonalEditingStyleBasedOnMedian<-function(data) {  # expecting attributes: "attacher" "editing.style"

  aggr = aggregate(data$editing.style,by=list(data$attacher),FUN=calculateEditFirstFraction)
  names(aggr)=c("attacher", "edit.first");

  aggr1 = aggregate(data$editing.style,by=list(data$attacher),FUN=calculateEditLastFraction)
  names(aggr1)=c("attacher", "edit.last");
  aggr=merge(aggr,aggr1);

  aggr2 = aggregate(data$editing.style,by=list(data$attacher),FUN=calculateEditThroughoutFraction)
  names(aggr2)=c("attacher", "edit.throughout");
  aggr=merge(aggr,aggr2);
  
  edit.dims=c("edit.first","edit.last","edit.throughout")
  for( i in 1:length(edit.dims) ) {    
    m=median(aggr[[edit.dims[i]]])
    if( m==0 ) {
	more=subset(aggr,aggr[[edit.dims[i]]]>median(aggr[[edit.dims[i]]]))
	less=subset(aggr,aggr[[edit.dims[i]]]<=median(aggr[[edit.dims[i]]]))
    } else {      
	more=subset(aggr,aggr[[edit.dims[i]]]>=median(aggr[[edit.dims[i]]]))
	less=subset(aggr,aggr[[edit.dims[i]]]<median(aggr[[edit.dims[i]]]))
    }	
    dim.category=paste(edit.dims[i],".category",sep="");
    more[[dim.category]]=paste("more.",edit.dims[i],sep="");
    less[[dim.category]]=paste("less.",edit.dims[i],sep="");
    combined=rbind(more,less);
    aggr=merge(aggr,combined);
    aggr[[dim.category]]=as.factor(aggr[[dim.category]]);
  }
  aggr
}

computePersonalEditingStyleBasedOnCounts=function(data) {

  aggr = aggregate(data$editing.style,by=list(data$attacher),
    FUN=function(x) { length(which(x=="edit.first")) })
  names(aggr)=c("attacher", "edit.first");
  aggr1 = aggregate(data$editing.style,by=list(data$attacher),
    FUN=function(x) { length(which(x=="edit.last"))})
  names(aggr1)=c("attacher", "edit.last");
  aggr=merge(aggr,aggr1);
  aggr2 = aggregate(data$editing.style,by=list(data$attacher),
    FUN=function(x) { length(which(x=="edit.throughout"))})
  names(aggr2)=c("attacher", "edit.throughout");
  aggr=merge(aggr,aggr2);

  aggr
}

############################ binning fractions of types of task ###################

computeFractionsOfTaskTypes<-function(data) {  # expecting attributes: "attacher" "editing.style"

  aggr = aggregate(data$bug.severity,by=list(data$attacher),
    FUN=function(x){length(which(x=="enhancement")) / length(x)})
  names(aggr)=c("attacher", "enhancement");

  aggr1 = aggregate(data$bug.severity,by=list(data$attacher),
    FUN=function(x){  length(which(x=="minor"))  / length(x)})
  names(aggr1)=c("attacher", "minor");
  aggr=merge(aggr,aggr1);

  aggr2 = aggregate(data$bug.severity,by=list(data$attacher),
    FUN=function(x){length(which(x=="normal")) / length(x)})
  names(aggr2)=c("attacher", "normal");
  aggr=merge(aggr,aggr2);

  task.dims=c("enhancement","minor","normal")
  for( i in 1:length(task.dims) ) {
    m=median(aggr[[task.dims[i]]]);
    print(m);
    more=subset(aggr,aggr[[task.dims[i]]]<=m)
    less=subset(aggr,aggr[[task.dims[i]]]>m)
    dim.category=paste(task.dims[i],".category",sep="");
    more[[dim.category]]=paste("more.",task.dims[i],sep="");
    less[[dim.category]]=paste("less.",task.dims[i],sep="");
    combined=rbind(more,less);
    aggr=merge(aggr,combined);
    aggr[[dim.category]]=as.factor(aggr[[dim.category]]);
  }
  aggr
}


computeTaskTypePerSubject=function(data) {


  aggr = aggregate(data$bug.severity,by=list(data$attacher),
    FUN=function(x){length(which(x=="enhancement")) })
  names(aggr)=c("attacher", "enhancement");

  aggr1 = aggregate(data$bug.severity,by=list(data$attacher),
    FUN=function(x){  length(which(x=="minor"))  })
  names(aggr1)=c("attacher", "minor");
  aggr=merge(aggr,aggr1);

  aggr2 = aggregate(data$bug.severity,by=list(data$attacher),
    FUN=function(x){length(which(x=="normal")) })
  names(aggr2)=c("attacher", "normal");
  aggr=merge(aggr,aggr2);

  aggr
}
######################### expert level by pre-determined criteria  #################################

computeExpertLevel <-function(data) {  # expecting attributes: "attacher" "attachment.id"
  aggr.col="number.of.traces";
  aggr = aggregate(data$attachment.id,by=list(data$attacher),FUN=function(x) { length(x)})
  names(aggr)[names(aggr)=="Group.1"]="attacher";
  names(aggr)[names(aggr)=="x"]=aggr.col;
  
  more = subset(aggr,aggr[[aggr.col]]>=median(aggr[[aggr.col]]));
  less = subset(aggr,aggr[[aggr.col]]<median(aggr[[aggr.col]]));
  more$expert.level = "more";  
  less$expert.level= "less";  
  combined=rbind(more,less);
  aggr=merge(aggr,combined);
  aggr$expert.level=as.factor(aggr$expert.level);
  
  aggr
}


#############################stats #######################################

anyCorrelationWithChiSq<- function(data,c1,c2) {
  print("chi sq between");
  print(c1);
  print(c2);
  tmp=data.frame(data[[c1]],data[[c2]])
  names(tmp)=c(c1,c2);
  print(table(tmp))
  chisq.test(table(tmp))

}

stResiduals2=function(chi.sq.result) { # from Laura Thomas's guide p.38
  o=chi.sq.result$observed;
  ni<-rowSums(o) # row sums
  nj<-colSums(o) # column sums
  n<-sum(o)   # total sample size
  resid.pear=chi.sq.result$residuals;
  n*resid.pear/sqrt(outer(n-ni,n-nj,"*") ) 
  # standardized Pearson residuals
}

stats=function(inputData,c1,c2) {
  #mosiacPlotForEditingStyles(inputData,c1,c2);
  (chi.sq.result=anyCorrelationWithChiSq(inputData,c1,c2))
  (st.residuals=stResiduals2(chi.sq.result))
  (p.values=2*pnorm(-abs(st.residuals)))
  print(chi.sq.result)
  print(st.residuals)
  print(p.values)
}

########################### qualitative analysis ##################################

getBugIds=function(selected.attachment.ids) {
  selected.bug.ids=c()
  for( i in 1:length(selected.attachment.ids )) {
      curr.bug.id=subset(data,data$attachment.id == selected.attachment.ids[i])$bug.id;
      print(curr.bug.id)     
      selected.bug.ids=c(selected.bug.ids, as.numeric(levels(curr.bug.id)[curr.bug.id]))
      print(selected.bug.ids);
  }
  selected.bug.ids;
}

getBugSeverities=function(selected.attachment.ids) {
  selected.bug.severities=c()
  for( i in 1:length(selected.attachment.ids )) {
      curr.bug.severity=subset(data,data$attachment.id == selected.attachment.ids[i])$bug.severity;
      print(curr.bug.severity)     
      selected.bug.severities=c(selected.bug.severities, levels(curr.bug.severity)[curr.bug.severity])
      print(selected.bug.severities);
  }
  selected.bug.severities;
}

getAttachers=function(selected.attachment.ids) {
  selected=c()
  for( i in 1:length(selected.attachment.ids )) {
      curr=subset(data,data$attachment.id == selected.attachment.ids[i])$attacher;
      print(curr)     
      selected=c(selected, levels(curr)[curr])
      print(selected);
  }
  selected;
}

getExpertLevels=function(selected.attachers) {
  selected=c()
  for( i in 1:length(selected.attachers )) {
      curr=subset(attacher.data,attacher.data$attacher == selected.attachers[i])$expert.level;
      print(curr)     
      selected=c(selected, levels(curr)[curr])
      print(selected);
  }
  selected;
}

loadBugProductComponentInformation <-function() {
  setwd("../analyze-using-probability-beginning-or-end");
  data<-read.csv(
                 file="bug-id-product-component.csv",
                 sep=",",head=TRUE,  strip.white = TRUE);
  data
}
