loadPeopleEditData <-function() {
  #setwd("/home/annie/workspaces/20100919-task-boundary/TaskBoundary/R");
  #setwd("../edit-style");
  data<-read.csv(
                 file="mylyn-output2.csv",
                 sep=",",head=TRUE);
  data
}

loadEvents<-function() {
  loadAndNormalizeTime();
}

loadAndNormalizeTime<-function() {
  data<-loadPeopleEditData();
  mins <- getMins(data);
  maxs <- getMaxs(data);
  normalizedData <-normalizeData(data,mins,maxs);
  normalizedData <- removeBeginningEnd(normalizedData);
  normalizedData
}

loadPeople<-function() {
  #setwd("../edit-style");
  ppls<-read.csv(
                 file="attachment-id-to-attacher.csv",
                 sep=",",head=TRUE);
  ppls
}

loadProjectData=function() {
  
  project.data=read.csv(file="bug-id-product-component.csv", sep=",",head=TRUE);
  project.data
}

histograms <-function(data,data2,filename,label1,label2) {

  png(filename, bg="white", height=400,width=600)
  par(mfrow=c(1,2),oma=c(2,2,6,2), mar=c(5, 5, 2, 1))

  numbers <- hist(data$start.date,plot=FALSE);  
  numbers$counts <- numbers$counts/1000;
  plot(numbers$mids,numbers$counts,
       col="blue", type="o",
       ylab="Frequency (in thousnads)", xlab="Start time of events", main="Start time of events");
  numbers2 <- hist(data2$start.date,plot=FALSE);
  numbers2$counts <- numbers2$counts/1000;  
  lines(numbers2$mids,numbers2$counts,type="o", pch=22, lty=2, col="red")
  legend("topleft", c(label1, label2), cex=0.8, bty="n",
         col=c("blue","red"), pch=21:22, lty=1:2);

  numbers <- hist(data$end.date,plot=FALSE);
  numbers$counts <- numbers$counts/1000;
  plot(numbers$mids,numbers$counts,
       col="blue", type="o",
       ylab="Frequency (in thousands)", xlab="End time of events", main="End time of events")
  numbers2 <- hist(data2$end.date,plot=FALSE);
  numbers2$counts <- numbers2$counts/1000;  
  lines(numbers2$mids,numbers2$counts,type="o", pch=22, lty=2, col="red")
  legend("topleft", c(label1, label2), cex=0.8, bty="n",
         col=c("blue","red"), pch=21:22, lty=1:2);

  mtext("Event timing within tasks", NORTH<-3, line=2, adj=0.5, cex=1.2, outer=TRUE);

  dev.off();
}

removeBeginningEnd<-function(data) {
  data<-subset(data,data$start.date<=1 & data$end.date >= 0);
#  histograms(subset(data,data$kind=="selection"),subset(data,data$kind=="edit"), "histogram.png", "selection","edit");
  
  # don't filter because that's just the way Mylyn collector is to aggregate events
 # data<-subset(data,data$start.date>=0.10 & data$start.date<=0.9); # trimmed to get rid of mylyn collection artifacts
#  editSelectionsTTest(data)
  data
}

getMins <-function(data) {
  aggr = aggregate(cbind(data$start.date),by=list(data$attachment.id),FUN=min)
  names(aggr)[names(aggr)=="Group.1"]="attachment.id";
  names(aggr)[names(aggr)=="V1"]="task.start";
  aggr
}

getMaxs <-function(data) {
  aggr = aggregate(cbind(data$end.date),by=list(data$attachment.id),FUN=max)
  names(aggr)[names(aggr)=="Group.1"]="attachment.id";
  names(aggr)[names(aggr)=="V1"]="task.end";
  aggr
}

normalizeData<-function(data,mins,maxs){
  merged<-merge(data,mins,by="attachment.id");
  merged<-merge(merged,maxs,by="attachment.id");

  merged$start.date <- (merged$start.date - merged$task.start) / (merged$task.end - merged$task.start);
  merged$end.date <- (merged$end.date - merged$task.start) / (merged$task.end - merged$task.start);

  merged
}

############################# old: construct edit vectors ##################################

loadEditVectors<-function() {
  print("loading edit events");
  editEvents<-loadAndNormalizeTime();

  print("making edit events into edit vectors");
  editVectors<-augmentWithQuartiles(editEvents,"attachment.id");
  editVectors
}

firstQuartile <-function(x) {
  quantile(x,c(0.25))[["25%"]];
}
thirdQuartile <- function(x) {
  quantile(x,c(0.75))[["75%"]];
}

augmentWithQuartiles <-function(data,aggregateOn) {

  firstQs <- aggregate(cbind(data$start.date),by=list(data[[aggregateOn]]),FUN=firstQuartile)
  names(firstQs)[names(firstQs)=="Group.1"]=aggregateOn;
  names(firstQs)[names(firstQs)=="V1"]="first.quartile";

  meds <-  aggregate(cbind(data$start.date),by=list(data[[aggregateOn]]),FUN=median)
  names(meds)[names(meds)=="Group.1"]=aggregateOn;
  names(meds)[names(meds)=="V1"]="median";

  thirdQs<- aggregate(cbind(data$start.date),by=list(data[[aggregateOn]]),FUN=thirdQuartile)
  names(thirdQs)[names(thirdQs)=="Group.1"]=aggregateOn;
  names(thirdQs)[names(thirdQs)=="V1"]="third.quartile";
  
  temp<-merge(firstQs,meds);
  temp<-merge(temp,thirdQs);
  temp
}

############################## old: cluster edit vectors ####################################

clusterTasksByEdits <- function(events) {
  print("loading the edit timings");
#  data<-loadAndNormalizeTime();
#  tasks<-augmentWithQuartiles(data,"attachment.id");
  tasks<-loadEditVectors();
  task.clusters<-clusterEditVectorsUsingPam(tasks,3);
  tasks
}

clusterEditVectors<-function(data,numberOfClusters) {
                                        # K-Means Cluster Analysis
  fit <- kmeans(data.frame(data$first.quartile,data$median,data$third.quartile), numberOfClusters)
                                      
  data <- data.frame(data, fit$cluster)  # append cluster assignment
  data$fit.cluster<-as.factor(data$fit.cluster)
                                        
#  library(cluster);
#  clusplot(data, fit$cluster, color=TRUE, shade=TRUE, labels=4, lines=0);  #labels=4 means only the ellipses are labeled
  data
}


clusterEditVectorsUsingPam<-function(data,numberOfClusters) {
  library(cluster);
  attrs <- names(data) %in% c("first.quartile","median","third.quartile") 
  d <-data[attrs]
  fit <- pam(daisy(d), 4)
  
#plot(fit) # for silhouette plot
  
  data <- data.frame(data, fit$cluster)  # append cluster assignment
  data$fit.cluster<-as.factor(data$fit.cluster)
  data
}

findOptimalNumberOfClusterForEditVectors<-function(data) {
  library(fpc);
  attrs <- names(data) %in% c("first.quartile","median","third.quartile") 
  d <-data[attrs]
  pamk(daisy(d));
  # This equals 4 when clustering on edit vectors
}

graphClusterEditVectors<-function(data,numberOfClusters) {

                                        # K-Means Cluster Analysis
  fit <- kmeans(data.frame(data$first.quartile,data$median,data$third.quartile), numberOfClusters)
                                      
  data <- data.frame(data, fit$cluster)  # append cluster assignment
                                        
  library(cluster);
  clusplot(data, fit$cluster, color=TRUE, shade=TRUE, labels=4, lines=0);  #labels=4 means only the ellipses are labeled
  fit
}

### old: personal edit style based on choosing the majority style from clustered  edit vectors ############

markPersonalEditStyle<-function(data) {
  aggr = aggregate(cbind(data$fit.cluster),by=list(data$attacher),FUN=majority)
  names(aggr)[names(aggr)=="Group.1"]="attacher";
  names(aggr)[names(aggr)=="V1"]="editing.style";
  result<-merge(data,aggr);
  result
}

majority<-function(x) {
  tbl<-table(x)
  m<-max(tbl)
  l<-length(tbl)
  result<- -1;
  for( i in 1:l ) {
    if( m==tbl[i] ) {
      result<-i;
    }
  }
  result
}

clusterPeopleByEdits <- function() {
  print("loading mapping from bug id to people commit id");
  data<-loadAndNormalizeTime();
  ppls<-loadPeople(data);
  ppls<-merge(ppls,data);
  ppls<-augmentWithQuartiles(ppls,"attacher");
  ppls<-clusterEditVectors(ppls,3);
  ppls

}

################################ cluster people by edit vectors #################################

computerEditStylePerTraceThroughClustering<-function(ppls) { # expecting loadPeople has been called
  tasks<-loadEditVectors();
  tasks<-merge(tasks,ppls)

  library(cluster);
 # attrs <- names(tasks) %in% c("first.quartile","median","third.quartile") 
  attrs <- names(tasks) %in% c("median") 
  d <-tasks[attrs]
  fit <- pam(daisy(d), 3)
  
  print("medoids");
  d[fit$medoids,]
  
  setwd("../analysis");
  plot(fit) # for silhouette plot
  dev.set(2);
  dev.print (device=postscript, file="silhouette-edit-cluster-median-only-n3.eps", 
    onefile=FALSE, paper="special", horizontal=FALSE);
  dev.off();

  aggr <- data.frame(task$attacher.id, fit$cluster)  # append cluster assignment
  names(tasks)[names(tasks)=="fit.cluster"]="editing.style";
  aggr$editing.style<-as.factor(tasks$editing.style)


  almostAllInClusters=function(x) { almostAllInCluster(x,c("1","2","3")) }

  aggr = aggregate(tasks$editing.style,by=list(tasks$attacher),FUN=almostAllInClusters)
  names(aggr)[names(aggr)=="Group.1"]="attacher";
  names(aggr)[names(aggr)=="x"]="personal.editing.style";
  aggr$personal.editing.style=as.factor(aggr$personal.editing.style)


  aggr
}


almostAllInCluster <-function(x,clusters) { 
  result="no.definite.style";
  for( i in 1:length(clusters)) {
    if( length(which(x==clusters[i])) > length(x)*0.5 ) {
      result=paste("mostly",clusters[i],sep=".");
    }
  }
  result
}
############################ binning by pre-determined criteria #################################

computeEditStylePerTrace <-function(events) {  # expecting attributes: "attacher" "kind" "attachment.id" "start.date"

  events<-subset(events,events$kind=="edit");

  aggr = aggregate(events$start.date,by=list(events$attachment.id),FUN=determineEditStyle)
  names(aggr)[names(aggr)=="Group.1"]="attachment.id";
  names(aggr)[names(aggr)=="x"]="editing.style";

  aggr$editing.style<-as.factor(aggr$editing.style);
  aggr
}

determineEditStyle <- function(edits) {    # expecting attributes "kind" "start.date", with the same attacher
  numberOfEditEvents=length(edits);
  numberOfEditEventsInFirstThird = length(which(edits <1/3))
  ratioOfEditsInFirstThird = numberOfEditEventsInFirstThird / numberOfEditEvents;
  result ="";

  if( ratioOfEditsInFirstThird < 1/3 ) {
    result = "edit.last";
  } else if (ratioOfEditsInFirstThird > 1/3 ) {
    result = "edit.first";
  } else {
    result = "other";
  }
  result
}

computeEditStylePerAttacher <-function(data) {  # expecting attributes: "attacher" "editing.style"

  aggr = aggregate(data$editing.style,by=list(data$attacher),FUN=almostAll)
  names(aggr)[names(aggr)=="Group.1"]="attacher";
  names(aggr)[names(aggr)=="x"]="personal.editing.style";

  aggr$personal.editing.style=as.factor(aggr$personal.editing.style);
  aggr
}

almostAll <-function(x) { 
  result="no.definite.style";
  if( length(which(x=="edit.first")) > length(x)*0.5 ) {
    result="mostly.edit.first";
  } else if( length(which(x=="edit.last")) > length(x)*0.5) {
    result="mostly.edit.last";
  }
  result
}



                                        
#  library(cluster);
#  clusplot(data, fit$cluster, color=TRUE, shade=TRUE, labels=4, lines=0);  #labels=4 means only the ellipses are labeled


#tasks<-clusterTasksByEdits()
#ppls<-clusterPeopleByEdits()
