# README: change the path to the direction that contains this file

setwd("/home/annie/workspaces/20100919-task-boundary/TaskBoundary/R/analyze-using-probability-beginning-or-end");
source("analyze-using-probability-beginning-or-end.R");
source("task-clustering.R");
source("edit-style.R");

print("loading tasks");
data<-loadTaskData();                    

print("add in attacher"); 
ppls<-loadPeople();
data<-merge(ppls,data);

print("what data to analysis?");
#c1="task.cluster";
c1="bug.severity";
#c2="personal.editing.style"
c2="editing.style"
#c2="expert.level"
mylyn= -1; # all
#mylyn = 0; # no mylyn
#mylyn = 1; # just mylyn 

print("loading events");
events<-merge(loadEvents(),data);

print("add in editing style");
traces =computePercentageOfEditsInBeginning(events);
editing.style.per.trace=computeEditingStyleBasedOnEditsInFirstHalf(traces);
data=merge(data,editing.style.per.trace,all.x=FALSE,all.y=FALSE)

# RQ 1
stats(data,c1,c2);
# RQ 2
stats(data, "contain.stack.trace", "editing.style")
table(data$contain.stack.trace,data$editing.style)
summary(data$contain.stack.trace)
summary(data$editing.style)
normal.data = subset(data, data$bug.severity=="normal")
stats(normal.data, "contain.stack.trace", "editing.style")

# For non-Mylyn projects:
# X-squared = 6.0973, df = 2, p-value = 0.04742
# effect sze: 0.06877502

########### effect size, not applicable
library(pwr)
total=sum(table(data[[c1]],data[[c2]]) )

# w index
phi=ES.w2( table(data[[c1]],data[[c2]])  / total) 
# 0.07570465
# 0.08142614 for just Mylyn

# cramer's V
V.phi=sqrt((phi^2)/(3-1)) # 0.048
V.chi.sq=sqrt(16.9618/ (total*(3-1))) # 0.048

# contingency coefficient
C=sqrt(16.9618/(16.9618+total)) 

ES.w2( table(data[["contain.stack.trace"]],data[["editing.style"]])  / total)

############

computeOdds = function(tbl,t) {
  p=(tbl[t,] / rowSums(tbl)[[t]])
  print(tbl)
  print(tbl[t,])
  print(rowSums(tbl)[[t]])
  print(p)
  odds= p / (1-p)
  print(odds)
  odds
}

computeOddsColumn = function(tbl,t) {
  p=(tbl[,t] / colSums(tbl)[[t]])
  print(tbl)
  print(tbl[,t])
  print(colSums(tbl)[[t]])
  print(p)
  odds= p / (1-p)
  print(odds)
  odds
}

## Enhancement tasks are negatively associated with \textsc{edit-first}
tbl=table(data[[c1]],data[[c2]])
odds=computeOdds(tbl,"enhancement")
odds[c(2,3)]/odds[1]
#       edit.last edit.throughout 
#     0.29151784      0.07732951 
#Reciprocals:       2.045445       11.191485 
## GREAT: Of all the enhancement tasks, 
## edit-first were 11 times less likely than edit.throughout
## and 2 times less likely than edit.last
odds.col=computeOddsColumn(tbl,"edit.first")
odds.col[1]/odds.col[c(2,3)]
## major-enhancement is more negative

## Minor tasks are positively associated with \textsc{edit-throughout}
odds=computeOdds(tbl,"minor")
odds[3]/odds[c(1,2)]
# edit.first  edit.last 
# 11.191485   5.471419 
## GREAT: Of all the minor tasks,
## edit-throughout is 11 times more likely than edit.first
## and 5 times more likely than edit.last
odds.col=computeOddsColumn(tbl,"edit.throughout")
odds.col[2]/odds.col[c(1,3)]
#enhancement      normal 
#  0.3779636   0.1121844 
## edit.throughout-normal & enhancement are more positive

## negatively associated with \textsc{edit-last}
odds[2]/odds[c(1,3)]
#      edit.first edit.throughout 
#      2.0454446       0.1827679 
odds.col=computeOddsColumn(tbl,"edit.last")
odds.col[c(1,3)]/odds.col[2]
# enhancement      normal 
# 0.28884102  0.07654626 
# Reciprical: 
#   2.645757    8.913894 
## GREAT: Of all the tasks with edit.last styles
## minor tasks are 2.6 times less likely than enhancements
## and 8.9 times less likely than normal bug fixes


## Major bug fixes are positively associated with \textsc{edit-first}
odds=computeOdds(tbl,"normal")
odds[1]/odds[c(2,3)]
#            edit.last edit.throughout 
#      0.4795572       0.1461745 
odds.col=computeOddsColumn(tbl,"edit.first")
odds.col[3]/odds.col[c(1,2)]
# enhancement       minor 
#   7.771959   15.757209 
## GREAT: Of all the edit-first tasks,
## major bugs are 7.8 times more likely than enhancements
## and 15.8 times more likely than minor bug fixes

##   negatively associated with \textsc{edit-throughout}
odds=computeOdds(tbl,"normal")
odds[3]/odds[c(1,2)]
# edit.first  edit.last 
#  6.841139   3.280717 
odds.col=computeOddsColumn(tbl,"edit.throughout")
odds.col[3]/odds.col[c(1,2)]
#enhancement       minor 
#   3.369128    8.913894 

#########################################
computeOddsAgresti=function(tbl,i,j,I,J) {
  odds=tbl[i,j]*tbl[I,J]/(tbl[I,j]*tbl[i,J])
  odds
}

## Enhancement tasks are negatively associated with \textsc{edit-first}
tbl=table(data[[c1]],data[[c2]])
computeOddsAgresti(tbl,1,1,2,2) 
125/341 / (69/124) # arranged in a different way: 124/341 / (69/125)
 # 0.6587615
computeOddsAgresti(tbl,1,1,3,2)
# 0.6526938
computeOddsAgresti(tbl,1,1,2,3)
# 0.8347542
computeOddsAgresti(tbl,1,1,3,3)
# 0.6234414
# For enhancements, the estimated odds of having 
# edit-first instead of edit-last style was 34.1% less than 
# the estimated odds for minor bug fixes,
# and 34.7% less if edit-first instead of edit-throughout.

# The estimated odds of having edit-first instead of edit-last style
# was 16.5% less than the odds for major bug fixes,
# and 37.7% less if edit-first instead edit-throughout.


## Minor tasks are positively associated with \textsc{edit-throughout}
computeOddsAgresti(tbl,3,2,1,1)#0.6526938
computeOddsAgresti(tbl,3,2,2,1)# 0.9907892
computeOddsAgresti(tbl,3,2,1,3)# 1.046921
computeOddsAgresti(tbl,3,2,2,3)# 1.326613
# For minor tasks, the estimated odds of having edit-throughout instead
# of edit-last style is 1.32 times the estimated odds for major bug fixes.

## negatively associated with \textsc{edit-last}
computeOddsAgresti(tbl,2,2,1,1)# 0.6587615
computeOddsAgresti(tbl,2,2,3,1)# 1.009296
computeOddsAgresti(tbl,2,2,1,3)# 0.7891683
computeOddsAgresti(tbl,2,2,3,3)# 0.7537994
# For minor tasks, the estimated odds of having edit-last instead of edit-first
# is 0.34 less than the estimated odds for enhancements.
0.238/0.132
0.29/0.106
# ==  For enhancement tasks, the estimated odds of having edit-first
# instead of edit-last is 0.34 less than the estimated odds for minor tasks.

# For minor tasks, the estimated odds of having 
# edit-last instead of edit-first is 0.21 less than,
# and edit-last instead of edit-throughout 0.25 less than,
# the estimated odds for major bug fixes.


## Major bug fixes are positively associated with \textsc{edit-first}
computeOddsAgresti(tbl,1,3,2,1)# 1.197957
computeOddsAgresti(tbl,1,3,2,2)# 0.7891683
computeOddsAgresti(tbl,1,3,3,1)#  1.604
computeOddsAgresti(tbl,1,3,3,2)# 1.046921
# For major bug fixes, the estimated odds of edit-first
# instead of edit-throughout is 1.60 times,
# and edit-frist instead of edit-last 1.20 times,
# the estimated odds of enhancements.

##   negatively associated with \textsc{edit-throughout}
computeOddsAgresti(tbl,3,3,1,1)#  0.6234414
computeOddsAgresti(tbl,3,3,1,2)# 0.955182
computeOddsAgresti(tbl,3,3,2,1)#   0.7468563
computeOddsAgresti(tbl,3,3,2,2)# 0.7537994
# For major bug fixes, the estimated odds of edit-throughout instead of 
# edit-first were 0.62 times the estimated odds for enhacements.
# The estimated odds of edit-throughout instead of edit last
# were 0.747 the estimated odds for enchancements
# and 0.753 the estimated odds for minor.